library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity magic_dev is
  port
  (
    csi_clk_clk : in std_logic;
    rsi_rst_reset : in std_logic;
    
    avs_s0_address       : in std_logic_vector(1 downto 0);
    avs_s0_read          : in std_logic;
    avs_s0_readdata      : out std_logic_vector(31 downto 0);
    avs_s0_readdatavalid : out std_logic;
    avs_s0_write         : in std_logic;
    avs_s0_writedata     : in std_logic_vector(31 downto 0)
  );
end entity;

architecture rtl of magic_dev is

  subtype slv32_t is std_logic_vector(31 downto 0);
  type aslv32_t is array(natural range <>) of slv32_t;

  signal regs : aslv32_t(0 to 3);

begin

  reg_read: process(csi_clk_clk, rsi_rst_reset)
    variable reg : integer range regs'range;
  begin
    if rsi_rst_reset = '1' then
      avs_s0_readdata <= (others => '0');
      avs_s0_readdatavalid <= '0';
    elsif rising_edge(csi_clk_clk) then
      reg := to_integer(unsigned(avs_s0_address));
      avs_s0_readdata <= regs(reg);
      avs_s0_readdatavalid <= avs_s0_read;
    end if;
  end process;

  reg_write: process(csi_clk_clk, rsi_rst_reset)
    variable reg : integer range regs'range;
  begin
    if rsi_rst_reset = '1' then
      regs <= (others => (others => '0'));
    elsif rising_edge(csi_clk_clk) then
      reg := to_integer(unsigned(avs_s0_address));
      if avs_s0_write = '1' then
        regs(reg) <= avs_s0_writedata;
      end if;
    end if;
  end process;

end;
